from setuptools import setup, find_packages

setup(
    name="pyisv",
    version="1.0.0",
    packages=find_packages(),
    install_requires=[
    ],
    author="Rama Sesha Sridhar Mantripragada",
    author_email="ram.sridhar4@gmail.com",
    description="A Machine Learning Technique to Extract the Intra Seasonal Variability (ISV) from the Geophysical Data.",
    url="https://github.com/ram-sridhar/pyisv",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
    ],
)
